# -*- coding: utf-8 -*-
#
# BitrateCalc Converter
#
# Coded by Dr.Best (c) 2013
# Support: www.dreambox-tools.info
# E-Mail: dr.best@dreambox-tools.info
#
# This plugin is open source but it is NOT free software.
#
# This plugin may only be distributed to and executed on hardware which
# is licensed by Dream Multimedia GmbH.
# In other words:
# It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
# to hardware which is NOT licensed by Dream Multimedia GmbH.
# It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
# on hardware which is NOT licensed by Dream Multimedia GmbH.
#
# If you want to use or modify the code or parts of it,
# you have to keep MY license and inform me about the modifications by mail.
#
#	USAGE EXAMPLE:
#	<widget backgroundColor="#080808" noWrap="1" font="Regular;17" foregroundColor="grey" render="Label" position="0,193" size="1280,20" source="session.CurrentService" transparent="1" zPosition="3" halign="center" valign="top">
#		<convert type="BitrateCalc"/>
#	</widget>
#
#
#
# 	REMARK: I dont think that this converter makes really sense in the Infobar, quite the opposite: your system could be slower than before... ;)



from Components.Converter.Converter import Converter
from enigma import iServiceInformation, iPlayableService, eTimer, iPlayableServicePtr
from Components.Element import cached
from Plugins.Extensions.BitrateViewer.bitratecalc import eBitrateCalculator

class sinBitrateCalc2(Converter, object):

	VIDEOBITRATE = 0
	AUDIOBITRATE = 1
	BITRATE = 2 # default

	def __init__(self, type):
		Converter.__init__(self, type)
		if type == "VideoBitrate":
			self.type = self.VIDEOBITRATE
		elif type == "AudioBitrate":
			self.type = self.AUDIOBITRATE
		else:
			self.type = self.BITRATE
		
		self.videoBitrate = None
		self.audioBitrate = None
		self.video_bitrate = self.audio_bitrate = 0

	def cleanUp(self):
		if self.videoBitrate is not None:
			self.videoBitrate.callback.remove(self.getVideoBitrateData)
			self.videoBitrate = None
			self.video_bitrate = 0
		if self.audioBitrate is not None:
			self.audioBitrate.callback.remove(self.getAudioBitrateData)
			self.audioBitrate = None
			self.audio_bitrate = 0
		Converter.changed(self, (self.CHANGED_CLEAR,))
		
	def doSuspend(self, suspended):
		if suspended:
			self.cleanUp()
		else:
			self.initializeBitrateCalc()

	def getServiceInfoValue(self, info, what, ref=None):
		v = ref and info.getInfo(ref, what) or info.getInfo(what)
		if v != iServiceInformation.resIsString:
			return None
		return ref and info.getInfoString(ref, what) or info.getInfoString(what)

	def initializeBitrateCalc(self):
		service = self.source.service
		if isinstance(service, iPlayableServicePtr):
			info = service and service.info()
			ref = None
			if info is not None:
				rref = self.getServiceInfoValue(info, iServiceInformation.sServiceref, ref)
				if rref is not None:
					vpid = apid = -1
					serviceInfo = service.info()
					vpid = serviceInfo.getInfo(iServiceInformation.sVideoPID)
					apid = serviceInfo.getInfo(iServiceInformation.sAudioPID)
					if vpid > 0 and (self.type == self.VIDEOBITRATE or self.type == self.BITRATE):
						self.videoBitrate = eBitrateCalculator(vpid, rref, 1000, 1024*1024)
						self.videoBitrate.callback.append(self.getVideoBitrateData)
					if apid > 0 and (self.type == self.AUDIOBITRATE or self.type == self.BITRATE):
						self.audioBitrate = eBitrateCalculator(apid, rref, 1000, 64*1024)
						self.audioBitrate.callback.append(self.getAudioBitrateData)

	@cached
	def getText(self):
		if self.type == self.BITRATE and self.videoBitrate is not None and self.audioBitrate is not None:
			return "V: %d kbit/s  A: %d kbit/s" % (self.video_bitrate, self.audio_bitrate)
		elif self.type == self.VIDEOBITRATE and self.videoBitrate is not None:
			return  "Video: %d kbit/s" % self.video_bitrate
		elif self.type == self.AUDIOBITRATE and self.audioBitrate is not None:
			return "Audio: %d kbit/s" % self.audio_bitrate
		else:
			return ""

	text = property(getText)

	def getVideoBitrateData(self, value, status): # value = rate in kbit/s, status ( 1  = ok || 0 = nok (zapped?))
		if status:
			self.video_bitrate = value
			Converter.changed(self, (self.CHANGED_POLL,))
		else:
			if self.videoBitrate is not None:
				self.videoBitrate.callback.remove(self.getVideoBitrateData)
				self.videoBitrate = None
			self.video_bitrate = 0
			Converter.changed(self, (self.CHANGED_CLEAR,))

	def getAudioBitrateData(self, value, status):
		if status:
			self.audio_bitrate = value
			Converter.changed(self, (self.CHANGED_POLL,))
		else:
			if self.audioBitrate is not None:
				self.audioBitrate.callback.remove(self.getAudioBitrateData)
				self.audioBitrate = None
			self.audio_bitrate = 0
			Converter.changed(self, (self.CHANGED_CLEAR,))

	def changed(self, what):
		if what[0] == self.CHANGED_SPECIFIC and what[1] in (iPlayableService.evEnd,):
			self.cleanUp()
